import{s as r,c as s,a as e,C as t}from"./DPfltzjH.js";import{_ as l}from"./CRcR2DqT.js";var d={parser:e,get db(){return new t},renderer:s,styles:r,init:l(a=>{a.class||(a.class={}),a.class.arrowMarkerAbsolute=a.arrowMarkerAbsolute},"init")};export{d as diagram};
